*  Input directory for SAS code and data - remember trailing slash;

%let INDIR=c:\workshop\ws136\;

*  Output directory for HTML and XML output - remember trailing slash;

%let OUTDIR=c:\inetpub\wwwroot\workshop\ws136\;

*  Web server directory for HTML and XML output - remember trailing slash;

%let WEBDIR=http://localhost/workshop/ws136/;

*  Library to store styles and tagsets;
 
libname myLib "&INDIR";
 
*  Library containing the sample data;
 
libname pharma "&INDIR" access=read;
 
*  Set the ODS search path for styles and tagsets;

ods path myLib.tmplmst(update)
         sashelp.tmplmst(read);

*;
*  Include recent versions of the MSOffice2K and
*  ExcelXP ODS tagsets.
*;

%inc "&INDIR.msoffice2k.sas";
%inc "&INDIR.excelxp.sas";

*;
*  Include the macro (XLXP2SAS) to import Excel XML to SAS tables.
*;

%inc "&INDIR.loadxl.sas";


*  == Moving SAS Data and Analytical Results to Excel and Word ==;

*;
*  First attempt at making an HTML file for Excel and Word.
*
*  Outcome: (1) Cell borders OK in Word and Excel
*           (2) Green background OK in Word, mapped to gray in Excel
*           (3) Dates formatted correctly in Word, different in Excel
*           (4) Leading zeroes retained in Word, dropped in Excel
*;

*  Use ODS to create the HTML file;

ods listing close;
ods tagsets.MSOffice2K style=Banker path="&OUTDIR" file='aedata.htm';
  title; footnote;
  proc print data=pharma.phcae noobs label;
    by protocol;
    var patient visit aedate aecode aetext aesev aesevc frequency;
  run; quit;

  proc tabulate data=pharma.phcae;
    by protocol;
    var aesev;
    class aetext aesevc;
    table aetext*aesevc,aesev*pctn;
    keyword all pctn;
    keylabel pctn='Percent';
  run; quit;
ods tagsets.MSOffice2K close;


*;
*  Second attempt at making an HTML file for Excel and Word.
*
*  Outcome: (1) Cell borders still OK in Word and Excel
*           (2) Green background OK in Word, still mapped to gray in Excel
*           (3) Dates are now formatted correctly in Excel
*           (4) Leading zeroes now retained in Excel
*;

*  Use ODS to create the HTML file, with style overrides applied;

ods listing close;
ods tagsets.MSOffice2K style=Banker path="&OUTDIR" file='aedata.htm';
  title; footnote;
  proc print data=pharma.phcae noobs label;
    by protocol;
    var patient visit;
    var aedate / style={htmlstyle="mso-number-format:ddmmmyyyy"};
    var aecode / style={htmlstyle="mso-number-format:00000000"};
    var aetext aesev aesevc frequency;
  run; quit;

  proc tabulate data=pharma.phcae;
    by protocol;
    var aesev;
    class aetext aesevc;
    table aetext*aesevc,aesev*pctn;
    keyword all pctn;
    keylabel pctn='Percent';
  run; quit;
ods tagsets.MSOffice2K close;


*  Use ODS to create a new style, changing only 'headerbg';

proc template;
  define style styles.MSOBanker;
    parent = styles.Banker;
    replace colors /
      'headerfgemph'   = cx033366
      'headerbgemph'   = cxFFFFCD
      'headerfgstrong' = cx033366
      'headerbgstrong' = cxFFFFCD
      'headerfg'       = cx033366
      'headerbg'       = cxCCFFCC
      'datafgemph'     = cx000000
      'databgemph'     = cxFFFFCD
      'datafgstrong'   = cx000000
      'databgstrong'   = cxFFFFCD
      'datafg'         = cx000000
      'databg'         = cxFFFFCD
      'batchfg'        = cx000000
      'batchbg'        = cxFFFFCD
      'tableborder'    = cx000000
      'tablebg'        = cx000000
      'notefg'         = cx033366
      'notebg'         = cxFFFFCD
      'bylinefg'       = cx033366
      'bylinebg'       = cxFFFFCD
      'captionfg'      = cx033366
      'captionbg'      = cxFFFFCD
      'proctitlefg'    = cx033366
      'proctitlebg'    = cxFFFFCD
      'titlefg'        = cx033366
      'titlebg'        = cxFFFFCD
      'systitlefg'     = cx033366
      'systitlebg'     = cxFFFFCD
      'Conentryfg'     = cx033366
      'Confolderfg'    = cx033366
      'Contitlefg'     = cx033366
      'link2'          = cx800080
      'link1'          = cx0000FF
      'contentfg'      = cx000000
      'contentbg'      = cxFFFFCD
      'docfg'          = cx033366
      'docbg'          = cxFFFFCD;
  end;
run; quit;


*;
*  Final attempt at making an HTML file for Excel and Word.
*
*  Outcome: (1) Cell borders still OK in Word and Excel
*           (2) Green background now OK in both Word and Excel
*           (3) Dates are still formatted correctly in Excel
*           (4) Leading zeroes still retained in Excel
*;

*  Use ODS to create the HTML file, with a new style and style overrides applied;

ods listing close;
ods tagsets.MSOffice2K style=MSOBanker path="&OUTDIR" file='aedata.htm';
  title; footnote;
  proc print data=pharma.phcae noobs label;
    by protocol;
    var patient visit;
    var aedate / style={htmlstyle="mso-number-format:ddmmmyyyy"};
    var aecode / style={htmlstyle="mso-number-format:00000000"};
    var aetext aesev aesevc frequency;
  run; quit;

  proc tabulate data=pharma.phcae;
    by protocol;
    var aesev;
    class aetext aesevc;
    table aetext*aesevc,aesev*pctn;
    keyword all pctn;
    keylabel pctn='Percent';
  run; quit;
ods tagsets.MSOffice2K close;


*;
*  First attempt at making an XML file for Excel.
*
*  Outcome: (1) Cell borders are missing
*           (2) Green background is OK due to MSOBanker style
*           (3) Leading zeroes are dropped
*;

*  Use ODS to create the XML file;

ods listing close;
ods tagsets.ExcelXP style=MSOBanker path="&OUTDIR" file='aedata.xml';
  title; footnote;
  proc print data=pharma.phcae noobs label;
    by protocol;
    var patient visit aedate aecode aetext aesev aesevc frequency;
  run; quit;

  proc tabulate data=pharma.phcae;
    by protocol;
    var aesev;
    class aetext aesevc;
    table aetext*aesevc,aesev*pctn;
    keyword all pctn;
    keylabel pctn='Percent';
  run; quit;
ods tagsets.ExcelXP close;


*;
*  Second attempt at making an XML file for Excel.
*
*  Outcome: (1) Cell borders are still missing
*           (2) Green background is still OK
*           (3) Leading zeroes are retained
*;

*  Use ODS to create the XML file;

ods listing close;
ods tagsets.ExcelXP style=MSOBanker path="&OUTDIR" file='aedata.xml';
  title; footnote;
  proc print data=pharma.phcae noobs label;
    by protocol;
    var patient visit aedate;
    var aecode / style={tagattr="\00000000"};
    var aetext aesev aesevc frequency;
  run; quit;

  proc tabulate data=pharma.phcae;
    by protocol;
    var aesev;
    class aetext aesevc;
    table aetext*aesevc,aesev*pctn;
    keyword all pctn;
    keylabel pctn='Percent';
  run; quit;
ods tagsets.ExcelXP close;


*;
*  Use ODS to create a new style.  The following values of borderwidth are supported:
*    1 = no border lines
*    2 = thin lines
*    3 = medium lines
*    4 = thick lines
*;

proc template;
  define style styles.XLBanker;
    parent = styles.MSOBanker;
    style Header from Header /
      borderwidth=2;
    style RowHeader from RowHeader /
      borderwidth=2;
    style Data from Data /
      borderwidth=2;
  end;
run; quit;


*;
*  Final attempt at making an XML file for Excel.
*
*  Outcome: (1) Cell borders are now present
*           (2) Green background is still OK
*           (3) Leading zeroes are still retained
*;

*  Use ODS to create the XML file;

ods listing close;
ods tagsets.ExcelXP style=XLBanker path="&OUTDIR" file='aedata.xml';
  title; footnote;
  proc print data=pharma.phcae noobs label;
    by protocol;
    var patient visit aedate;
    var aecode / style={tagattr="\00000000"};
    var aetext aesev aesevc frequency;
  run; quit;

  proc tabulate data=pharma.phcae;
    by protocol;
    var aesev;
    class aetext aesevc;
    table aetext*aesevc,aesev*pctn;
    keyword all pctn;
    keylabel pctn='Percent';
  run; quit;
ods tagsets.ExcelXP close;


*  == Moving Excel Data to SAS ==;


*  Import the Excel data from local file system to SAS tables;

%xlxp2sas(excelfile=&OUTDIR.mydata.xml,
          mapfile=&INDIR.excelxp.map);

*  Import the Excel data from a Web server to SAS tables;

filename myxml URL "&WEBDIR.mydata.xml';

%xlxp2sas(excelfile=fileref:myxml,
          mapfile=&INDIR.excelxp.map);